use crate::{commands::utils, Context, Error};
use cached::proc_macro::cached;
use chrono::{DateTime, Utc};
use log::warn;
use reqwest::header::AUTHORIZATION;
use serde::Deserialize;
use serde_json::{
    Map,
    Value::{self, Object},
};
use std::fs;

async fn autocomplete_command<'a>(
    _ctx: Context<'_>,
    partial: &'a str,
) -> impl Iterator<Item = String> + 'a {
    let res: Vec<String> = get_coms()
        .keys()
        .filter(move |e| e.contains(&partial))
        .map(|e| e.to_string())
        .collect();
    res.into_iter()
}

fn get_coms() -> Map<String, Value> {
    let path = "./data/commands.json";
    let data = fs::read_to_string(path).expect("Unable to read file");
    let res: serde_json::Value = serde_json::from_str(&data).expect("Unable to parse");
    if let Object(things) = res {
        return things;
    } else {
        return Map::<String, Value>::new();
    }
}

// F1TV links throughout the seasons
#[poise::command(slash_command)]
pub async fn coms(
    ctx: Context<'_>,
    #[description = "Which command to execute?"]
    #[autocomplete = "autocomplete_command"]
    command: String,
) -> Result<(), Error> {
    match get_coms().get(&command) {
        None => {
            ctx.say(format!("Command `{}` not found", command)).await?;
        }
        Some(com) => {
            ctx.say(com.as_str().unwrap()).await?;
        }
    }

    Ok(())
}
